/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JasmineEventSFBImpl.java 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.beans.impl;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.ow2.easybeans.osgi.annotation.OSGiResource;
import org.ow2.jasmine.event.beans.JasmineEventEB;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventSFBRemote;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemoryPool;
import org.ow2.util.log.LogFactory;

/**
 * Bean to insert facts into the working memories.
 * @author Loris Bouzonnet
 */
@Stateless
@Remote(JasmineEventSFBRemote.class)
public class JasmineEventSFBImpl implements JasmineEventSFBRemote {

    /**
     * Bundle context that will be injected by the container.
     */
    @OSGiResource
    private BundleContext bundleContext = null;

    private ServiceListener workingMemoryPoolListener;

    private IDroolsWorkingMemoryPool droolsWorkingMemoryPool;

    private ServiceReference  workingMemoryPoolReference;

    /**
     * Bean startup call-back method.
     */
    @PostConstruct
    public void start() {
        workingMemoryPoolListener = new ServiceListener() {

            public void serviceChanged(final ServiceEvent event) {
                ServiceReference ref = event.getServiceReference();
                // React on REGISTERED|UNREGISTERING events
                switch(event.getType()) {
                case ServiceEvent.REGISTERED:
                    droolsWorkingMemoryPool = (IDroolsWorkingMemoryPool) bundleContext.getService(ref);
                    workingMemoryPoolReference = ref;
                    break;
                case ServiceEvent.UNREGISTERING:
                    droolsWorkingMemoryPool = null;
                    workingMemoryPoolReference = null;
                    break;
                }

            }
        };
        try {
            bundleContext.addServiceListener(
                    workingMemoryPoolListener, "(objectClass=IDroolsWorkingMemoryPool)");
        } catch (InvalidSyntaxException e) {
            LogFactory.getLog(getClass()).error("Cannot add a listener for the working memory pool", e);
        }
        workingMemoryPoolReference =
            bundleContext.getServiceReference(IDroolsWorkingMemoryPool.class.getName());
        if(workingMemoryPoolReference != null) {
            droolsWorkingMemoryPool = (IDroolsWorkingMemoryPool) bundleContext.getService(workingMemoryPoolReference);
        }
    }

    /**
     * Bean stop call-back method.
     */
    @PreDestroy
    public void stop() {
        if(workingMemoryPoolListener != null) {
            bundleContext.removeServiceListener(workingMemoryPoolListener);
        }
        if(workingMemoryPoolReference != null) {
            bundleContext.ungetService(workingMemoryPoolReference);
        }
    }

    public void saveEvent(final JasmineEventEB e) {
        if(droolsWorkingMemoryPool != null) {

        	// FIXME Inserting in all working memories. Filters should be set
        	// to redirect events to their real target.
            droolsWorkingMemoryPool.insertAll(e);
        }
    }

}
